using System;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Channels;
using System.Threading;

using KnownObjects;
using Belikov.GenuineChannels;
using Belikov.GenuineChannels.BroadcastEngine;
using Belikov.GenuineChannels.Security;

namespace Server
{
	/// <summary>
	/// EventProviderImplementation.
	/// </summary>
	public class EventProviderImplementation : MarshalByRefObject, IExchangeEventProvider
	{
		public EventProviderImplementation()
		{
			// bind server's methods
			this._dispatcher.BroadcastCallFinishedHandler += new BroadcastCallFinishedHandler(this.BroadcastCallFinishedHandler);
			this._dispatcher.CallIsAsync = true;
			IBroadcastSenderProvider iBroadcastSenderProvider = (IBroadcastSenderProvider) ChannelServices.GetChannel("IPMulticastSender");
			this._dispatcher.Add(iBroadcastSenderProvider.GetBroadcastSender("/Test/EventReceiver"));

			_timer = new Timer(new TimerCallback(this.TimerCallback), null, 333, 333);
		}

		/// <summary>
		/// A timer to fire an event.
		/// </summary>
		private static Timer _timer;

		/// <summary>
		/// Chat members.
		/// </summary>
		private Dispatcher _dispatcher = new Dispatcher(typeof(IExchangeEventReceiver));

		/// <summary>
		/// Subscribes to an event.
		/// </summary>
		public void Subscribe(IExchangeEventReceiver iExchangeEventReceiver)
		{
			this._dispatcher.Add((MarshalByRefObject) iExchangeEventReceiver);
			Console.WriteLine("A client has been registered ({0}).", GenuineUtility.CurrentRemoteHost.PhysicalAddress.ToString());
		}

		/// <summary>
		/// Returns a marshal-by-ref object.
		/// </summary>
		public MarshalByRefObject GetAnyMbr()
		{
			return this;
		}

		private int messageNumber = 0;

		/// <summary>
		/// Fires the event three times per second.
		/// </summary>
		/// <param name="state">Useless parameter.</param>
		private void TimerCallback(object state)
		{
			// request encryption and compression
			SecuritySessionParameters securitySessionParameters = new SecuritySessionParameters("/exchange/sk", SecuritySessionAttributes.EnableCompression, TimeSpan.MinValue);
			using(new SecurityContextKeeper(securitySessionParameters))
			{
				IExchangeEventReceiver iExchangeEventReceiver = (IExchangeEventReceiver) this._dispatcher.TransparentProxy;
				iExchangeEventReceiver.ReceiveEvent("A message N" + Interlocked.Increment(ref this.messageNumber));
			}
		}

		/// <summary>
		/// Called by broadcast dispatcher when all calls are performed.
		/// Does not undertake any actions.
		/// </summary>
		/// <param name="dispatcher">Source dipatcher.</param>
		/// <param name="message">Source message.</param>
		/// <param name="resultCollector">Call results.</param>
		public void BroadcastCallFinishedHandler(Dispatcher dispatcher, IMessage message, ResultCollector resultCollector)
		{
			Console.WriteLine("Message was sent. Number of failed clients: {0}", resultCollector.Failed.Count);
		}

		/// <summary>
		/// This is to insure that when created as a Singleton, the first instance never dies,
		/// regardless of the expired time.
		/// </summary>
		/// <returns></returns>
		public override object InitializeLifetimeService()
		{
			return null;
		}
	}
}
